// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Permissive License.
// See http://www.microsoft.com/resources/sharedsource/licensingbasics/sharedsourcelicenses.mspx.
// All other rights reserved.


Type.registerNamespace('FontSize');

FontSize.FontSizeBehavior = function(element) {

    FontSize.FontSizeBehavior.initializeBase(this, [element]);

    // the DOM elemens we'll be interacting with
    //
    this._sizePanelElement = null;
    this._incrementElement = null;
    this._decrementElement = null;
    this._hoverCssClass = null;
    
    // the behaviors we create
    //
    this._popupBehavior = null;
    this._hoverBehavior = null;
    
    // Handler delegates
    //
    this._hoverHandler = null;
    this._unhoverHandler = null;    
    this._incrementHandler = null;
    this._decrementHandler = null;
    
    // property values
    //    
    this._fontSizeIncrement = 2;    
    this._maxFontSize = 28;
    this._minFontSize = 4;    
    
    // state bits
    //
    this._fontSize = 0;    
    this._popupVisible = false;
    this._fontUnit = "px"; 
}

FontSize.FontSizeBehavior.prototype = {

    initialize : function() {
        FontSize.FontSizeBehavior.callBaseMethod(this, 'initialize');

        // setup the hover behavior
        //
        if (this._hoverCssClass && this._sizePanelElement) {
            
            this._hoverHandler = Function.createDelegate(this, this._onTargetHover);       
            this._unhoverHandler = Function.createDelegate(this, this._onTargetUnhover);
            
            this._hoverBehavior = $create(AjaxControlToolkit.HoverBehavior, {unhoverDelay:200, hoverElement: this._sizePanelElement}, null, null, this.get_element());
            this._hoverBehavior.add_hover(this._hoverHandler);
            this._hoverBehavior.add_unhover(this._unhoverHandler);     
        }        
        
        // setup the popup behavior
        //
        if (this._sizePanelElement) {
           
            this._popupBehavior = $create(AjaxControlToolkit.PopupBehavior, 
                    {id:  this.get_id() + "_Popup", // make the ID derive from our ID for uniqueness
                    positioningMode:AjaxControlToolkit.PositioningMode.TopRight, 
                    parentElement: this.get_element()}, 
                    null, null, this._sizePanelElement);                        
        }
               
        
        // set up our handlers for the +/- behavior
        //
        if (this._incrementElement) {
            this._incrementHandler = Function.createDelegate(this, this._onIncrementFontSize);
            $addHandler(this._incrementElement, "click", this._incrementHandler);
        }
        
        if (this._decrementElement) {
            this._decrementHandler = Function.createDelegate(this, this._onDecrementFontSize);
            $addHandler(this._decrementElement, "click", this._decrementHandler);
        } 
        
        // check client state
        //
        var clientState = this.get_ClientState();
        if (clientState) {
            var stateItems = clientState.split(":");
            
            if (stateItems.length) {                
                this._fontUnit = stateItems[1];
                this._setFontSize(parseInt(stateItems[0]));
            }
        }      
    },

    dispose : function() {
        
        if (this._hoverBehavior) {
            this._hoverBehavior.dispose();
            this._hoverBeahvior = null;            
        }
        
        if (this._popupBehavior) {
            this._popupBehavior.dispose();
            this._hoverBeahvior = null;            
        }

        FontSize.FontSizeBehavior.callBaseMethod(this, 'dispose');
    },
    
    _getCurrentFontSize : function() {
    
        // this function figures out the current font size of the element.
        //
        
        // first it checks the style.  if it's in points or pixels, it uses that value.
        //                
        if (!this._fontSize) {
            var e = this.get_element();
            var currentSize = AjaxControlToolkit.CommonToolkitScripts.getCurrentStyle(e, "fontSize");
                        
            if (currentSize) {
                var index = currentSize.indexOf("px");
                
                if (index > 0) {
                    currentSize = parseInt(currentSize.substr(0, index));                    
                    this._fontUnit = "px";
                }          
                else {                                    
                    index = currentSize.indexOf("pt");
                    
                    if (index > 0) {
                        this._fontUnit = "pt";
                        currentSize = parseInt(currentSize.substr(0, index));                                            
                    } 
                    else {
                        currentSize = 0;
                    }                   
                }     
            }
            
            // if we couldn't parse the value, try to compute it.
            //
            if (!currentSize) {
                // we couldn't ocme up wiht a size size we like., try to figure out how big it is in pixels.
                // just create and measure an element with just text, then remove it.
                //                
                var dummyElement = document.createElement("span");
                dummyElement.innerHTML = "XYZ";                
                e.appendChild(dummyElement);
                currentSize = dummyElement.offsetHeight;
                e.removeChild(dummyElement);
            }
            this._fontSize = currentSize;            
        }
        
        return this._fontSize;        
    },    
    
    
    set_MinFontSize : function(val) {
        this._minFontSize = val;
    },
    
    get_MinFontSize : function() {
        return this._minFontSize;
    },
    
    get_MaxFontSize : function() {
        return this._maxFontSize;
    },
    
    set_MaxFontSize : function(val) {
        this._maxFontSize = val;
    },
    
    _setFontSize : function(size) {
        // helper function for setting the font size
        //
        this._fontSize = size;
        
        // append the units.
        //
        this.get_element().style.fontSize = size + this._fontUnit;
        
        // set the clientState
        //
        this.set_ClientState(size + ":" + this._fontUnit);
    },
    
    _onDecrementFontSize : function(eventArgs) {
        var fontSize = this._getCurrentFontSize();
        
        if (fontSize > this.get_MinFontSize()) {
            fontSize -= this._fontSizeIncrement;
        }
        this._setFontSize(fontSize);
        eventArgs.preventDefault();
    },
    
    _onIncrementFontSize : function(eventArgs) {
        var fontSize = this._getCurrentFontSize();
        
        if (fontSize < this.get_MaxFontSize()) {
            fontSize += this._fontSizeIncrement;
        }        
        this._setFontSize(fontSize);
        eventArgs.preventDefault();
    },
    
   
    _onTargetHover : function(eventArgs) {
        
        var e = this.get_element();
        
        Sys.UI.DomElement.addCssClass(e, this._hoverCssClass);
        
        if (this._popupBehavior && !this._popupVisible) {            
            this._popupVisible = true;
            
            this._stopAnimations();
            
            // call show to make the the popup visible
            //         
            this._popupBehavior.show();
            
            // now animate it's opacity
            //
            var anim = $create(AjaxControlToolkit.Animation.FadeInAnimation,
                {target: this._sizePanelElement, duration: .25, minimumOpacity: 0, maximumOpacity:.75});                
            
            // we cache the running animation so we can cancel it if we need to hide.
            //
            this._sizePanelElement._runningAnimation = anim;    
             
            var handler = Function.createDelegate(this,
                function() {                    
                    this._sizePanelElement._runningAnimation = null;                        
                });
             
            anim.add_ended(handler);    
                
            anim.play();     
        }            
    },
    
    _onTargetUnhover : function(eventArgs) {
        var e = this.get_element();
        Sys.UI.DomElement.removeCssClass(e, this._hoverCssClass);
        
        if (this._popupBehavior && this._popupVisible) {
            this._popupVisible = false; 
            
            
            // make sure another animation isn't already running.
            //
            this._stopAnimations();
                                 
            var anim = $create(AjaxControlToolkit.Animation.FadeOutAnimation,
                {target: this._sizePanelElement, duration: .15, minimumOpacity: 0, maximumOpacity:.75});                
             
            this._sizePanelElement._runningAnimation = anim;       
             
            var handler = Function.createDelegate(this,
                function() {  
                
                    // clear out our state.
                    //                    
                    this._sizePanelElement._runningAnimation = null;                                          
                    this._popupVisibile = false;
                    this._popupBehavior.hide();
                });
             
            anim.add_ended(handler);
             
            anim.play();            
        }
    },
    
    _stopAnimations : function() {
    
        // stop any running animation
        //
        if (this._sizePanelElement && this._sizePanelElement._runningAnimation) {
            this._sizePanelElement._runningAnimation.stop();
            this._sizePanelElement._runningAnimation = null;
        }   
    },

    get_hoverCssClass : function() {
        return this._hoverCssClass;
    },
    
    set_hoverCssClass: function(val) {
        this._hoverCssClass = val;
    },
    
    get_incrementElement : function() {
        return this._incrementElement;
    },
    
    set_incrementElement: function(val) {
        this._incrementElement = val;
    },
    
    get_decrementElement : function() {
        return this._decrementElement;
    },
    
    set_decrementElement: function(val) {
        this._decrementElement = val;
    },
    
    get_sizePanelElement : function() {
        return this._sizePanelElement;
    },
    
    set_sizePanelElement: function(val) {
        this._sizePanelElement = val;
    },
     
    get_FontSizeIncrement : function() {
        return this._fontSizeIncrement;
    },
    
    set_FontSizeIncrement: function(val) {
        this._fontSizeIncrement = val;
    }
}

FontSize.FontSizeBehavior.registerClass('FontSize.FontSizeBehavior', AjaxControlToolkit.BehaviorBase);
